#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy(wxnacy@gmail.com)
# Description:

import click

class Logger():
    def __init__(self, *args, **kwargs):
        pass

    def info(self, *msg):
        color = 'cyan'
        message = ' '.join(msg)
        click.echo(click.style('[INFO] ', fg=color) + message)

    def debug(self, *msg, with_color=False):
        color = 'yellow'
        message = ' '.join(msg)
        if with_color:
            message = click.style(message, fg=color)
        click.echo(click.style('[DBUG] ', fg=color) + message)

    def error(self, *msg, with_color=False):
        color = 'red'
        message = ' '.join(msg)
        if with_color:
            message = click.style(message, fg=color)
        click.echo(click.style('[ERRO] ', fg=color) + message)

    def warn(self, *msg, with_color=False):
        color = 'magenta'
        message = ' '.join(msg)
        if with_color:
            message = click.style(message, fg=color)
        click.echo(click.style('[WARN] ', fg=color) + message)






#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy(wxnacy@gmail.com)
# Description:

import configparser
from goss.app import utils


class Config():
    def __init__(self, filename=None):
        self.filename = filename

        self.config = configparser.ConfigParser()
        if filename:
            self.config.read(filename)

    def get_value(self, section, name):
        '''获取值'''

        if section not in self.config:
            return None

        sec = self.config[section]

        if name not in sec:
            return None

        return sec[name]


goss_config = Config(utils.GOSS_CONFIG_PATH)




#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy(wxnacy@gmail.com)
# Description:

from requests.auth import HTTPBasicAuth
from goss.app.logger import Logger
import json
import requests
import base64
import time
import os
import configparser
import sys
import mimetypes

logger = Logger()

API_URL = 'https://api.github.com{}'
ACCEPT = 'application/vnd.github.v3+json'

class BaseObject(object):
    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)

    def to_dict(self):
        return json.loads(self.to_json())

    def to_json(self):
        return json.dumps(self, default=lambda o: o.__dict__, sort_keys=True)

    def __str__(self):
        return self.to_json()

class Content(BaseObject):
    pass

class Author():
    def __init__(self, name, email):
        self.name = name
        self.email = email

class Response(BaseObject):
    def __init__(self):
        self.contents = []
    pass

class Github():
    debug = False
    def __init__(self, user, password):
        self.user = user
        self.password = password

    def set_author(self, name, email):
        self.author = Author(name, email)

    def set_owner(self, owner):
        self.owner = owner

    def set_debug(self, debug: bool):
        self.debug = debug

    def get_user_info(self):
        '''
        获取用户信息
        '''
        url = '/user'
        self._request('get', url)
        return self.response.status_code, self.response.json()

    def get_release(self):
        return Release(self)

    def get_file(self, owner, repository, path, ref='master'):
        '''
        获取文件
        https://developer.github.com/v3/repos/contents/#get-contents
        '''
        url = '/repos/{}/{}/contents/{}'.format(owner, repository, path)
        self._request('get', url, ref=ref)
        #  is_success = self.response.status_code == 200
        return self.response.status_code, self.response.json()

    def get_file_blob(self, owner, repository, sha, ref='master'):
        '''
        获取文件内容，超过 1 MB 时
        https://developer.github.com/v3/git/blobs/#get-a-blob
        '''
        url = '/repos/{}/{}/git/blobs/{}'.format(owner, repository, sha)
        self._request('get', url, ref=ref)
        return self.response.status_code, self.response.json()

    def delete_file(self, owner, repository, path, sha, branch='master'):
        '''
        删除文件
        https://developer.github.com/v3/repos/contents/#delete-a-file
        '''
        url = '/repos/{}/{}/contents/{}'.format(owner, repository, path)
        data = {
            "message": "Delete {}".format(path),
            "committer": {
                "name": self.author.name,
                "email": self.author.email
                },
            "sha": sha,
            "branch": branch
        }
        self._request('delete', url, **data)
        return self.response.status_code, self.response.json()

    def create_file_from_url(self, owner, repository, url, path=None, sha=None):
        '''
        从 url 创建文件
        '''
        res = requests.get(url)
        byte_data = res.content

        return self._create_file(owner, repository, url, byte_data, path, sha)

    def create_file_from_path(self, owner, repository, filepath, path=None, sha=None):
        '''
        从本地创建文件
        https://developer.github.com/v3/repos/contents/#create-a-file
        '''
        byte_data = None
        with open(filepath, 'rb') as f:
            byte_data = f.read()
            f.close()
        return self._create_file(owner, repository, filepath, byte_data, path, sha)

    def _create_file(self, owner, repository, filepath, byte_data,
            path = None, sha=None):
        '''
        创建文件
        https://developer.github.com/v3/repos/contents/#create-a-file
        '''
        if not path:
            path = os.path.basename(filepath)
        content = base64.b64encode(byte_data).decode()
        url = '/repos/{}/{}/contents/{}'.format(owner, repository, path)
        data = {
            "message": "Create file {}".format(path),
            "author": {
                "name": self.author.name,
                "email": self.author.email
                },
            "content": content
            }
        if sha:
            data['sha'] = sha
        self._request('put', url, **data)
        return self.response.status_code, self.response.json()

    def create_repository(self, name, description="", homepage="",
            private=False, has_issues = True, has_projects = True,
            has_wiki = True, **kw
        ):
        '''
        创建仓库 https://developer.github.com/v3/repos/#create
        '''
        all_args = locals()
        url = '/user/repos'
        all_args.pop('self')
        self._request('post', url, **all_args)
        return self.response.status_code, self.response.json()

    def create_organization_repository(self, organization, name,
            description="", homepage="", private=False, has_issues = True,
            has_projects = True, has_wiki = True, **kw
        ):
        '''
        创建组织仓库 https://developer.github.com/v3/repos/#create
        '''
        all_args = locals()
        url = '/orgs/{}/repos'.format(organization)
        all_args.pop('self')
        self._request('post', url, **all_args)
        success = self.response.status_code == 201
        return success, self.response.json()

    def delete_repository(self, owner, repo):
        '''
        删除 repository
        https://developer.github.com/v3/repos/#delete-a-repository
        '''
        url = '/repos/{}/{}'.format(owner, repo)
        self._request('delete', url)
        code = self.response.status_code
        return code, self.response.json() if code != 204 else None

    def get_owner_repositorys(self):
        '''
        获取当前用户的 repository 列表
        https://developer.github.com/v3/repos/#list-your-repositories
        '''
        url = '/user/repos'
        self._request('get', url)
        return self.response.status_code, self.response.json()

    def get_repository(self, owner, repo):
        '''
        获取 repository 详情
        https://developer.github.com/v3/repos/#get
        '''
        url = '/repos/{}/{}'.format(owner, repo)
        self._request('get', url)
        return self.response.status_code, self.response.json()

    def _request(self, method, url, **kw):
        data = {}
        if method in ('put', 'post', 'delete'):
            data['json'] = kw
        elif method in ('get'):
            data['params'] = kw

        req_data = dict(
            method = method,
            url = API_URL.format(url),
            headers={"Accept": ACCEPT},
            auth=HTTPBasicAuth(self.user, self.password),
        )
        if data:
            req_data.update(data)
        res = requests.request(**req_data)
        self.response = res
        if self.debug:
            logger.debug("Response code {}".format(res.status_code))
            if res.status_code != 204:
                logger.debug("Response data\n{}".format(
                    json.dumps(self.response.json(), indent=4)))

class Release(Github):
    def __init__(self, github, *args, **kwargs):
        self.github = github

    def create_release(self,owner, repo, tag_name, name="", body="",
            draft=False, prerelease=False, target_commitish='master'):
        '''
        创建 release
        https://developer.github.com/v3/repos/releases/#create-a-release
        '''
        if not body:
            body = 'Create by [goss](https://github.com/wxnacy/goss)'
        all_args = locals()
        all_args.pop('owner')
        all_args.pop('repo')
        all_args.pop('self')
        url = f'/repos/{owner}/{repo}/releases'
        self.github._request('post', url, **all_args)
        return self.github.response.status_code, self.github.response.json()

    def get_release(self, owner, repo, release_id):
        '''
        Get a single release
        https://developer.github.com/v3/repos/releases/#get-a-single-release
        '''
        url = f'/repos/{owner}/{repo}/releases/{release_id}'
        self.github._request('get', url)
        return self.github.response.status_code, self.github.response.json()

    def get_release_by_tag(self, owner, repo, tag_name):
        '''
        Get a release by tag name
        https://developer.github.com/v3/repos/releases/#get-a-release-by-tag-name
        '''
        url = f'/repos/{owner}/{repo}/releases/tags/{tag_name}'
        self.github._request('get', url)
        return self.github.response.status_code, self.github.response.json()

    def get_latest_release(self, owner, repo):
        '''
        Get the latest release
        https://developer.github.com/v3/repos/releases/#get-the-latest-release
        '''
        url = f'/repos/{owner}/{repo}/releases/latest'
        self.github._request('get', url)
        return self.github.response.status_code, self.github.response.json()

    def get_releases(self, owner, repo):
        '''
        List releases for a repository
        https://developer.github.com/v3/repos/releases/#list-releases-for-a-repository
        '''
        url = f'/repos/{owner}/{repo}/releases'
        self.github._request('get', url)
        return self.github.response.status_code, self.github.response.json()

    def delete_release(self, owner, repo, release_id):
        '''
        Delete a release
        https://developer.github.com/v3/repos/releases/#delete-a-release
        '''
        url = f'/repos/{owner}/{repo}/releases/{release_id}'
        self.github._request('delete', url)
        data = None
        if self.github.response.status_code != 204:
            data = self.github.response.json()
        return self.github.response.status_code, data

    def get_assets(self, owner, repo, release_id):
        '''
        List releases for a repository
        https://developer.github.com/v3/repos/releases/#list-releases-for-a-repository
        '''
        url = f'/repos/{owner}/{repo}/releases/{release_id}/assets'
        self.github._request('get', url)
        return self.github.response.status_code, self.github.response.json()

    def get_asset(self, owner, repo, asset_id):
        '''
        Get a single release asset
        https://developer.github.com/v3/repos/releases/#get-a-single-release-asset
        '''
        url = f'/repos/{owner}/{repo}/releases/assets/{asset_id}'
        self.github._request('get', url)
        return self.github.response.status_code, self.github.response.json()

    def delete_asset(self, owner, repo, asset_id):
        '''
        Delete a release asset
        https://developer.github.com/v3/repos/releases/#delete-a-release-asset
        '''
        url = f'/repos/{owner}/{repo}/releases/assets/{asset_id}'
        self.github._request('delete', url)
        data = None
        if self.github.response.status_code != 204:
            data = self.github.response.json()
        return self.github.response.status_code, data

    def upload_asset_from_path(self, owner, repo, release_id, path, name='',
            label=''):
        '''
        创建 asset
        https://developer.github.com/v3/repos/releases/#upload-a-release-asset
        '''
        if not name:
            name = os.path.basename(path)
        url = f'https://uploads.github.com/repos/{owner}/{repo}/releases/{release_id}/assets?name={name}&label={label}'
        data = open(path, 'rb').read()
        content_type = mimetypes.guess_type(path)[0]
        req_data = dict(
            method = 'POST',
            data = data,
            url = url,
            headers={"Accept": ACCEPT, "Content-Type": content_type},
            auth=HTTPBasicAuth(self.github.user, self.github.password),
        )
        res = requests.request(**req_data)
        return res.status_code, res.json()


if __name__ == "__main__":
    config_path = '{}/.config/gos/config'.format(os.getenv("HOME"))
    credential_path = '{}/.config/gos/credentials'.format(os.getenv("HOME"))

    credential = configparser.ConfigParser()
    credential.read(credential_path)
    default_cred = credential['default']

    g = Github(default_cred['user'], default_cred['password'])
    g.debug =True

    config = configparser.ConfigParser()
    config.read(config_path)
    author = config['user']
    g.set_author(author['name'], author['email'])

    owner = config['repository']['owner']
    repo = config['repository']['repo']
    #  g.create_from_file('image', '/Users/wxnacy/Downloads/mac-font2.png',
            #  'common/{}'.format(time.time()))

    #  g.create_repository('test2')
    #  g.create_organization_repository(, 'test3')
    #  g.get_file('wxnacy', 'image', 'push')
    #  g.get_owner_repositorys()

    #  g.upload_release_asset()

    r = Release(g)
    code, msg = r.get_releases('wxnacy', 'test')
    r1 = r.upload_asset_from_path('wxnacy', 'test', msg[0]['id'],
            '/Users/wxnacy/Downloads/404-1.png', '', '')
    #  print(r.create_release('wxnacy', 'test', 'v1.0.2' ))
    #  print(r.get_release_by_tag('wxnacy', 'test', 'v1.0.0'))
    #  print(r.get_release_by_id('wxnacy', 'test', 17763761))
    #  code, msg = r.get_assets('wxnacy', 'test', 17763761)
    #  print(r.get_asset('wxnacy', 'test', 13013362))
    #  print(r.delete_asset('wxnacy', 'test', msg[0]['id']))
    #  print(r.get_latest_release('wxnacy', 'test'))
    #  print(r1)
    #  r.delete_release('wxnacy', 'test', msg[0]['id'])

    #  code, msg = r.get_assets('wxnacy', 'test', 17763761)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy(wxnacy@gmail.com)
# Description:

import click
from goss import Github
from goss import __version__
from goss.app import utils
from goss.app.config import goss_config
from goss.app.logger import Logger
import configparser
import os
import pyperclip
import json
import base64
import requests

GOSS_CONFIG_HOME = '{}/.config/goss'.format(os.getenv("HOME"))
GOSS_CONFIG_PATH = '{}/config'.format(GOSS_CONFIG_HOME)
GOSS_CREDENTIAL_PATH = '{}/credentials'.format(GOSS_CONFIG_HOME)
GOSS_USER_INFO_PATH = '{}/user.json'.format(GOSS_CONFIG_HOME)

logger = Logger()

def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo(__version__.__version__)
    ctx.exit()

@click.group()
@click.option('--version', '-v', is_flag=True, callback=print_version,
              expose_value=False, is_eager=True, help='Show the version')
@click.option('--debug', '-d', is_flag=True, default=False, help='Run in debug')
@click.pass_context
def run(ctx, debug):
    '''
    Github Object Storage System
    '''
    if not os.path.exists(GOSS_CONFIG_HOME):
        os.mkdir(GOSS_CONFIG_HOME)
    if os.path.exists(GOSS_CREDENTIAL_PATH):
        cred = configparser.ConfigParser()
        cred.read(GOSS_CREDENTIAL_PATH)
        secs = cred.sections()
        if 'default' in secs:
            default_cred = cred['default']
            user = default_cred['user']
            password = default_cred['password']

            g = Github(user, password)
            g.set_owner(default_cred['owner'])

            conf = configparser.ConfigParser()
            conf.read(GOSS_CONFIG_PATH)
            author = conf['user']
            g.set_author(author['name'], author['email'])
            g.set_debug(debug)
            ctx.obj = g

def init_credentials_path():
    if not os.path.exists(GOSS_CONFIG_HOME):
        os.mkdir(GOSS_CONFIG_HOME)


@run.command(help='Log in to the github account')
@click.option('--user', '-u', prompt='Your Github user', help = 'Your Github user')
@click.option('--password', '-p', prompt='Your Github password',
        help = 'Your Github password',hide_input=True)
@click.option('--yes', '-y', is_flag = True, default = False, help = 'All questions answered yes')
@click.pass_context
def login(ctx, user, password, yes):
    g = ctx.obj
    confirm = utils.make_error_msg('The login information already exists. Do you want to replace it?')
    if not yes:
        if g and not click.confirm(confirm):
            return
    conf = configparser.ConfigParser()
    conf['default'] = dict(user = user, password = password)

    g = Github(user, password)
    code, data = g.get_user_info()
    if code != 200:
        utils.print_failed()
        logger.error('用户名或密码错误')
        return

    owner = data.get("name")
    conf['default']['owner'] = owner
    with open(GOSS_CREDENTIAL_PATH, 'w') as f:
        conf.write(f)
    with open(GOSS_USER_INFO_PATH, 'w') as f:
        f.write(json.dumps(data, indent = 4))
        f.flush()
        f.close()

    conf_data = dict(name = owner)
    email = data.get("email")
    if email:
        conf_data['email'] = email

    utils.config(GOSS_CONFIG_PATH, 'user', **conf_data)

    utils.print_success()
    logger.info('Name  :', owner)
    logger.info('Email :', email)

LARGE_SIZE = 1 * 1024 * 1024
TOO_LARGE_SIZE = 100 * 1024 * 1024


@run.command()
@click.option('--method', '-m', default = 'GET',
    help = 'GET/POST/PUT/DELETE for repository. Default is GET')
@click.option('--orga', '-o', help = 'If want create organization repository. Is required')
@click.option('--download', '-D', is_flag = True, help = 'Download file')
@click.option('--yes', '-y', is_flag = True, default = False, help = 'All questions answered yes')
@click.option('--output', '-O', help = 'Download name. Default is file name')
@click.option('--repo', '-r', required = True, help = 'Repository name')
@click.argument('path', default="/")
@click.pass_context
def file(ctx, repo, path, orga, method, download, output, yes):
    '''
    Get/Delete/Download your file

    If you want to upload file. Please use command

        goss <filepath> --repo=<repository-name>

    More usage see : goss --help
    '''
    g = ctx.obj
    if not g:
        click.echo('You have not logged in yet, please run goss-cli to login')
        ctx.exit()

    owner = orga if orga else g.owner

    def _get_progress(progress, total):
        '''打印进度条'''
        progress_ratio = progress / total
        progress_len = 20
        progress_num = int(progress_ratio * 20)
        pro_text = '[{:-<20s}] {:.2f}% {} / {}'.format(
            '=' * progress_num, progress_ratio * 100, progress, total)
        return pro_text


    def _save(p, filepath, content):
        '''保存文件'''
        byte_data = base64.b64decode(content.encode())
        _save_by_bytes(p, filepath, byte_data)

    def _save_by_bytes(p, filepath, byte_data):
        '''保存文件'''
        filedir = os.path.dirname(filepath)
        filename = os.path.dirname(filepath)
        with open(name, 'wb') as f:
            total = len(byte_data)
            progress = 0
            step = 10
            while progress < total:
                b = progress
                e = b + step
                f.write(byte_data[b:e])
                progress += step
                end = '\r'
                if progress >= total:
                    end = '\n'
                    progress = total
                print(p, _get_progress(progress, total), end = end)
            f.flush()
            f.close()

    if download:
        logger.info('Download file')
        logger.info('Owner\t: {}'.format(owner))
        logger.info('Repo\t: {}'.format(repo))
        logger.info('Path\t: {}'.format(path))
        logger.info('Waiting...')
        code, data = g.get_file(owner, repo, path)
        if code == 403:         # 文件太大，需要使用其它方式进行下载
            logger.warn('The file is larger than 1 MB and needs to be downloaded using the download_url.')
            path_dir = os.path.dirname(path)
            if not path_dir:
                path_dir = '/'
            c, files = g.get_file(owner, repo, path_dir)
            down_url = ''
            file_name = []
            for f in files:
                if f['path'] == path:
                    down_url = f['download_url']
                    file_name = f['name']
            file_res = requests.get(down_url)

            name = output or file_name
            _save_by_bytes(path, name, file_res.content)
            utils.print_success()
            ctx.exit()

        if code != 200:         # 其他错误直接返回
            utils.print_failed()
            logger.error(data.get("message"))
            ctx.exit()
        type = data.get("type")
        name = output or data.get("name")
        if isinstance(data, dict):
            content = data.get("content")
            _save(path, name, content)
        elif isinstance(data, list):
            pass
        utils.print_success()
        ctx.exit()

    def print_data(data):
        '''打印文件信息'''
        if isinstance(data, list):      # 打印列表
            utils.print_list(data,
                ['type\t', 'size\t', 'path\t\t', 'download_url'],
                ['\t', '\t', '\t\t', '']
            )
        elif isinstance(data, dict):    # 打印单个文件
            utils.print_dict(data, exclude=['_links'])
            pyperclip.copy(data['download_url'])
            logger.info('Now you can use download_url with {}.'.format(
                click.style('<CTRL-V>', fg='blue')
            ))

    def _del_file(path, sha):
        '''删除单个文件'''
        code, data = g.delete_file(owner, repo, path, sha)
        if code != 200:
            utils.print_failed()
            logger.error(path, filedata.get("message"))
            ctx.exit()
        logger.info(path, 'deleted')

    method = method.lower()
    if method == 'get':     # 处理 get 请求
        logger.info('Query file')
        logger.info('Owner\t: {}'.format(owner))
        logger.info('Repo\t: {}'.format(repo))
        logger.info('Path\t: {}'.format(path))
        logger.info('Waiting...')
        code, data = g.get_file(owner, repo, path)
        if code == 200:
            print_data(data)
            utils.print_success()
        else:
            utils.print_failed()
            logger.error(data.get("message"))
    elif method == 'delete':
        logger.info('Delete file')
        logger.info('Owner\t: {}'.format(owner))
        logger.info('Repo\t: {}'.format(repo))
        logger.info('Path\t: {}'.format(path))
        logger.info('Waiting...')
        code, filedata = g.get_file(owner, repo, path)
        if code != 200:
            utils.print_failed()
            logger.error(filedata.get("message"))
            ctx.exit()

        if isinstance(filedata, list):      # 删除文件夹

            if not yes:                     # 确认是否删除
                if not click.confirm('The {} is a folder, are you sure to delete?'.format(path)):
                    ctx.exit()
            else:
                logger.warn('The {} is a folder and is now being deleted'.format(path),
                        with_color=True)

            for f in filedata:
                _del_file(f['path'], f['sha'])

        elif isinstance(filedata, dict):    # 删除文件
            sha = filedata.get("sha")
            _del_file(path, sha)

        utils.print_success()


@run.command()
@click.option('--method', '-m', default = 'GET',
    help = 'GET/POST/PUT/DELETE for repository. Default is GET')
@click.option('--orga', '-o', help = 'If want create organization repository. Is required')
@click.argument('name', required = False)
@click.pass_context
def repo(ctx, name, orga, method):
    '''
    Get/Create your repositorys
    '''
    g = ctx.obj
    if not g:
        click.echo('You have not logged in yet, please run goss-cli to login')
        ctx.exit()

    owner = orga if orga else g.owner

    #  def print_repo_detail(r):
        #  '''打印单个 repository'''
        #  #  click.secho('*' * 60, fg='yellow')
        #  id_text = click.style('Id', fg='red')
        #  click.echo('{}     : {}'.format(utils.make_error_msg('Id'), r['id']))
        #  click.echo('{}   : {}'.format(utils.make_error_msg('Name'), r['name']))
        #  click.echo('{}  : {}'.format(utils.make_error_msg('Owner'),
            #  r['owner']['login']))
        #  click.echo('{}    : {}'.format(utils.make_error_msg('Url'),
            #  r['html_url']))
        #  click.echo('{}   : {}'.format(utils.make_error_msg('Desc'),
            #  r['description']))
        #  #  click.echo('{} : {}'.format('Detail', r['url']))
        #  click.echo('More details see: {}'.format(click.style(r['url'], fg='blue')))
        #  #  click.secho('*' * 60, fg='yellow')

    method = method.lower()
    if method == 'get':     # 处理 get 请求
        if name:            # 获取单个 repository
            logger.info('Query repository {}/{}'.format(owner, name))
            logger.info('Waiting...')
            code, repo = g.get_repository(owner, name)
            if code != 200:
                utils.print_failed()
                logger.error(repo.get("message"))
                ctx.exit()
            #  print_repo_detail(repo)
            utils.print_dict(repo, ['owner'])
            utils.print_success()
            ctx.exit()

        # 获取全部 repositorys
        logger.info('Query your repositorys.')
        logger.info('Waiting...')
        code, repos = g.get_owner_repositorys()
        if code != 200:
            utils.print_failed()
            logger.error(repo.get("message"))
            ctx.exit()
        utils.print_list(repos, 
            ['id\t\t', 'name\t', 'full_name\t', 'url'],
            ['\t', '\t', '\t', ''],
                )
        utils.print_success()
        ctx.exit()
    elif method == 'post':
        logger.info("Create repository")
        logger.info("Url : https://github.com/{}/{}".format(owner, name))
        logger.info('Waiting...')
        code, data = g.create_repository(name)
        if code != 201:
            utils.print_failed()
            logger.error(data.get("message"))
        logger.info("Create README.md")
        logger.info("Url : https://github.com/{}/{}/blob/master/README.md".format(owner, name))
        logger.info('Waiting...')
        code, data = g.create_file_from_url(owner, name,
            'https://raw.githubusercontent.com/wxnacy/goss/master/create_readme.md',
            'README.md')
        if code != 201:
            utils.print_failed()
            logger.error(data.get("message"))
        utils.print_success()


    elif method == 'delete':
        logger.info("Delete repository")
        logger.info("Url : https://github.com/{}/{}".format(owner, name))
        logger.info('Waiting...')
        code, data = g.delete_repository(owner, name)
        if code == 204:
            utils.print_success()
        else:
            utils.print_failed()
            logger.error(data.get("message"))


@run.command()
@click.argument('name', required = False)
@click.argument('value', required = False)
@click.pass_context
def config(ctx, name, value):
    '''
    Get/Create/Update goss config
    '''
    if name and value and '.' in name:
        names = name.split('.')
        section = names[0]
        key = names[1]
        utils.config(GOSS_CONFIG_PATH, section, **{key: value})
        utils.print_success()
        return

    conf = configparser.ConfigParser()
    conf.read(GOSS_CONFIG_PATH)
    secs = conf.sections()
    for sec in secs:
        click.echo('[{}]'.format(sec))
        kv = conf[sec]
        for k, v in kv.items():
            click.echo('    {} = {}'.format(k, v))

@run.command()
@click.option('--method', '-m', default = 'GET',
    help = 'GET/POST/PUT/DELETE for repository. Default is GET')
@click.option('--orga', '-o', help = 'If want create organization repository. Is required')
@click.option('--download', '-D', is_flag = True, help = 'Download file')
@click.option('--yes', '-y', is_flag = True, default = False, help = 'All questions answered yes')
@click.option('--output', '-O', help = 'Download name. Default is file name')
@click.option('--repo', '-r', help = 'Repository name. Default use config repo.name')
@click.argument('id', default="")
@click.pass_context
def release(ctx, repo, id, orga, method, download, output, yes):
    '''
    Get/Create/Delete/Download your Release/Asset

    More usage see : goss --help
    '''
    g = ctx.obj
    if not g:
        click.echo('You have not logged in yet, please run goss-cli to login')
        ctx.exit()

    r = g.get_release()
    owner = orga if orga else g.owner
    repo = repo if repo else goss_config.get_value('repo', 'name')

    def _get():
        if id:
            logger.info(f"Query release: {id}")
            logger.info(f"Owner:\t{owner}")
            logger.info(f"Repo:\t{repo}")
            logger.info("Waiting...")
            code, data = r.get_release(owner, repo, id)
            if code != 200:
                utils.print_failed()
                logger.error(repo.get("message"))
                ctx.exit()
            data.pop('author')
            utils.print_dict(data)
        else:
            logger.info("Query releases")
            logger.info(f"Owner:\t{owner}")
            logger.info(f"Repo:\t{repo}")
            logger.info("Waiting...")
            code, data = r.get_releases(owner, repo)
            if code != 200:
                utils.print_failed()
                logger.error(repo.get("message"))
                ctx.exit()
            utils.print_list(data,
                ['id\t\t', 'name\t', 'tag_name'],
                ['\t', '\t', ''],
            )
            logger.info('Now you can see the detail with command:')
            logger.info('\t\tgoss-cli release <id>')
        utils.print_success()
        ctx.exit()

    method_function = {
        'get': _get
    }
    method = method.lower()
    method_function[method]()



if __name__ == "__main__":
    run()
    #  credentials()



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy(wxnacy@gmail.com)
# Description:

import click
from goss import Github
from goss import __version__
from goss.app import utils
#  from goss.app import run
from goss.app.logger import Logger
import configparser
import os
import pyperclip
import json
import base64
import requests


#  @run.run.command()
@click.option('--method', '-m', default = 'GET',
    help = 'GET/POST/PUT/DELETE for repository. Default is GET')
@click.option('--orga', '-o', help = 'If want create organization repository. Is required')
@click.option('--download', '-D', is_flag = True, help = 'Download file')
@click.option('--yes', '-y', is_flag = True, default = False, help = 'All questions answered yes')
@click.option('--output', '-O', help = 'Download name. Default is file name')
@click.option('--repo', '-r', required = True, help = 'Repository name')
@click.argument('path', default="/")
@click.pass_context
def release(ctx, repo, path, orga, method, download, output, yes):
    '''
    Get/Create/Delete/Download your Release/Asset

    More usage see : goss --help
    '''
    g = ctx.obj
    if not g:
        click.echo('You have not logged in yet, please run goss-cli to login')
        ctx.exit()
    print('ww')

    #  method = method.lower()

    #  method_function = {
        #  'get': 
            #  }

    #  owner = orga if orga else g.owner

    #  method_function[method](owner, repo)

    #  def _get():





#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy(wxnacy@gmail.com)
# Description:

import click
import configparser
import os
import gevent
import time


GOSS_CONFIG_HOME = '{}/.config/goss'.format(os.getenv("HOME"))
GOSS_CONFIG_PATH = '{}/config'.format(GOSS_CONFIG_HOME)
GOSS_CREDENTIAL_PATH = '{}/credentials'.format(GOSS_CONFIG_HOME)

def print_success(msg='Success!'):
    click.secho(msg, fg='cyan')

def print_failed(msg='Failed!'):
    click.secho(msg, fg='red')

def print_progress(msg='Waiting...'):
    click.secho(msg, fg='yellow')

def print_error(msg):
    click.secho(msg, fg='red')

def make_progress_msg(msg):
    return click.style(msg, fg = 'yellow')

def make_error_msg(msg):
    return click.style(msg, fg = 'red')

def print_dict(data, exclude=[], key_fg='magenta', val_fg=''):
    item = dict(data)
    for e in exclude:
        item.pop(e)
    max_len = max([len(k) for k, v in item.items()])
    max_len += 10
    click.echo('')
    for k, v in item.items():
        key = click.style(k.title(), fg=key_fg).ljust(max_len)
        value = click.style(f"{v}", fg=val_fg)
        line = f"{key}: {value}"
        click.echo(line)

def print_list(data, include, val_suffixs):
    click.echo("")
    title = ''.join(include).title()
    hor_len = len(title.replace('\t', '-' * 8))
    hor_line = click.style('-' * hor_len, fg='yellow')
    click.secho(title, fg='magenta')
    click.secho(hor_line)
    for d in data:
        lines = []
        for i in range(len(include)):
            f = include[i]
            field_len = len(f.replace('\t', '*' * 8))
            field_key = f.rstrip('\t')
            field_val = f"{d[field_key]}{val_suffixs[i]}"
            lines.append(field_val)
        line = ''.join(lines)
        click.echo(line)
    click.secho(hor_line)
    total = click.style(str(len(data)), fg='cyan')
    click.secho(f'Total : {total}')
    click.echo('')




def config(filepath, section, **data):
    conf = configparser.ConfigParser()
    if os.path.exists(filepath):
        conf.read(filepath)

    val = conf[section]
    if val:
        data.update(val)

    conf[section] = data

    with open(filepath, 'w') as f:
        conf.write(f)
        f.close()

is_done = False

def print_dynamic_progress(msg = 'Waiting...'):
    print('begin progress')
    gevent.sleep(0)
    count = 0
    pl = ('⠸' ,'⠼' ,'⠴' ,'⠦' ,'⠧' ,'⠏' ,'⠙' ,'⠹')
    while count < 10000:
        for p in pl:
            print(f'{p} {msg}', end='\r')
        count += 1
    print('Success!       ')

def work():
    print('begin work')
    gevent.sleep(0)
    time.sleep(2)
    is_done = True
    print('end work')

class Progress():
    is_done = False
    is_waiting = True

    def print_dynamic_progress(self, msg = 'Waiting...'):
        print('begin progress')
        pl = ('⠸' ,'⠼' ,'⠴' ,'⠦' ,'⠧' ,'⠏' ,'⠙' ,'⠹')
        while not self.is_done:
            print(self.is_done, self.is_waiting)
            while self.is_waiting:
                for p in pl:
                    print(msg)
                    print(f'{p} {msg}', end='\r')
        print('Success!       ')

if __name__ == "__main__":
    #  gevent.joinall([
        #  gevent.spawn(work),
        #  gevent.spawn(print_dynamic_progress),
    #  ])
    #  print_dynamic_progress()
    lines = [
        dict(id = 17764214, name='vname', tag_name='v1.0.1'),
        dict(id = 17764214, name='', tag_name='v1.0.1'),
    ]
    #  print_list(lines, ['id\t\t', 'name\t', 'tag_name'])
    print_list(lines, 
                ['id\t\t', 'name\t', 'tag_name'],
                ['\t', '\t', ''],
            )





#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy(wxnacy@gmail.com)
# Description:

import click
from goss import Github
from goss.app import utils
from goss.app.config import goss_config
from goss.app.logger import Logger
import configparser
import os
import pyperclip
import sys

logger = Logger()

github = None
owner = None
repo = None

def init_git_config():
    '''初始化 git 信息'''
    credential_path = utils.GOSS_CREDENTIAL_PATH

    config_path = utils.GOSS_CONFIG_PATH

    if not os.path.exists(credential_path):
        return None, ''

    credential = configparser.ConfigParser()
    credential.read(credential_path)
    default_cred = credential['default']

    g = Github(default_cred['user'], default_cred['password'])

    config = configparser.ConfigParser()
    config.read(config_path)
    author = config['user']
    g.set_author(author['name'], author['email'])

    global owner
    owner = credential['default']['owner']

    return g

def create(filepath, path=None, repo=None, yes=False, **kw):
    '''
    创建文件
    '''
    g = init_git_config()
    r = g.get_release()
    if not path:
        conf_path = goss_config.get_value('repo', 'path')
        if conf_path:
            path = conf_path

    if not path:
        path = os.path.basename(filepath)

    if path.endswith('/'):
        path += os.path.basename(filepath)

    config = configparser.ConfigParser()
    config.read(utils.GOSS_CONFIG_PATH)
    if not repo:
        repo = config['repo']['name']

    def _upload_file():

        logger.info('Upload file')
        logger.info('Source\t:', filepath)
        logger.info('Path\t:', path)
        download_url = f'https://raw.githubusercontent.com/{owner}/{repo}/master/{path}'
        html_url = f'https://github.com/{owner}/{repo}/blob/master/{path}'
        logger.info('HtmlUrl\t:', html_url)
        logger.info('DownUrl\t:', click.style(download_url, fg='blue'))
        pyperclip.copy(download_url)
        logger.info('Now you can use it with {} and wait for the upload to succeed.'.format(
            click.style('<CTRL-V>', fg='blue')
        ))
        logger.info('Waiting...')

        if filepath.startswith('http'):
            default_create = g.create_file_from_url     # 通过 url 上传
        else:
            default_create = g.create_file_from_path    # 通过文件上传

        code, data = default_create(owner, repo, filepath, path)    # 上传
        if code == 404:
            utils.print_failed()
            logger.error('Repository Owner or Name not found', with_color=True)
            return

        # 如果文件已经存在，则询问是否覆盖
        if code == 422 and data.get("message") == \
                'Invalid request.\n\n"sha" wasn\'t supplied.':
            if not yes:
                if not click.confirm('This file is exists. Do you want to replace?'):
                    return
            else:
                logger.warn('This file already exists and is now replaced',
                        with_color=True)

            logger.info('Waiting...')
            # 遇到重复文件名，需要获取 sha 值后进行覆盖
            code, data = g.get_file(owner, repo, path)
            sha = data['sha']
            code, data = default_create(owner, repo, filepath, path, sha)

            if code != 200:
                utils.print_failed()
                utils.print_error(data['message'])
                sys.exit(0)

    def _upload_asset():
        name = kw.get("name")
        if not name:
            name = os.path.basename(filepath)
        logger.info('Upload asset')
        logger.info('Release\t:', release)
        logger.info('Source\t:', filepath)
        #  logger.info('Path\t:', path)
        download_url = f'https://github.com/{owner}/{repo}/releases/download/{release}/{name}'
        html_url = f'https://github.com/{owner}/{repo}/blob/master/{path}'
        #  logger.info('HtmlUrl\t:', html_url)
        logger.info('DownUrl\t:', click.style(download_url, fg='blue'))
        pyperclip.copy(download_url)
        logger.info('Now you can use it with {} and wait for the upload to succeed.'.format(
            click.style('<CTRL-V>', fg='blue')
        ))
        logger.info('Waiting...')
        code, data = r.get_release_by_tag(owner, repo, release)
        if code == 404:
            utils.print_failed()
            logger.error('Release not found', with_color=True)
            sys.exit(0)

        release_id = data.get("id")

        code, data = r.upload_asset_from_path(owner, repo, release_id,
            filepath, name
        )
        if code != 201:
            utils.print_failed()
            #  print(data)
            utils.print_error(data['message'])
            for err in data.get('errors', []):
                utils.print_error(f"\t- {err['field']} {err['code']}")
            sys.exit(0)

    release = kw.get('release')
    if release:
        _upload_asset()
    else:
        _upload_file()
    utils.print_success()

@click.command()
@click.option('--repo', '-r', help='Github repository name')
@click.option('--path', '-p', help='Github repository file path')
@click.option('--name', '-n', help='Want to upload asset name')
@click.option('--release', '-R', help='Github repository release name, If give it means upload a release asset.')
@click.option('--yes', '-y', is_flag = True, default = False, help = 'All questions answered yes')
#  @click.argument('filepath', type=click.Path(exists=True))
@click.argument('filepath')
def run(filepath, path, repo, yes, release, name):
    '''
    Github Object Storage System
    '''
    kw = locals()
    #  print(kw)
    create(**kw)
    pass

if __name__ == "__main__":
    run()
